#!/bin/sh
########################################################################
# File:           setup/ldap-server.sh                                 #
# myLinux Server: Copyright (c) 2003 Michael Oberg                     #
# Version:        0.92                                                 #
# Author:         Michael Oberg <michael.oberg@mylinuxproject.de>      #
#                                                                      #
# This program is free software; you can redistribute it and/or modify #
# it under the terms of the GNU General Public License as published by #
# the Free Software Foundation; either version 2 of the License, or    #
# (at your option) any later version.                                  #
#                                                                      #
# This program is distributed in the hope that it will be useful,      #
# but WITHOUT ANY WARRANTY; without even the implied warranty of       #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         #
# GNU General Public License for more details.                         #
#                                                                      #
# You should have received a copy of the GNU Public License along      #
# with this package; if not, write to the Free Software Foundation,    #
# Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.       #
########################################################################
source mylinux.conf
TOPDOMAIN=`echo $DOMAIN | sed 's/.*\.\([^.]*\)$/\1/'`
SUBDOMAIN=`echo $DOMAIN | sed 's/\(.*\)\.[^.]*$/\1/'`
##############################################################
# slapd.conf (ldap server configuration)                     #
##############################################################
# Requires: the ldap server has to be stopped before running,
# and ldap-db.sh or ldap-reset-db.sh has to be run before the
# ldap server could be restarted
# Consequences: the ldap directory is the central authentication
# instance; sendmail, squirrelmail, samba and the ldap client
# (ldap-client.sh) have to be reconfigured after each
# change of the ldap settings (except password changings)
cat << EOF | sed -e "s/dc=mylinux,dc=de/dc=$SUBDOMAIN,dc=$TOPDOMAIN/g" -e "s/rootpw[^ \t]*.*$/rootpw\t\t$LDAPPASSWD/" > /etc/openldap/slapd.conf
# : pkg/ldap/servers/slapd/slapd.conf,v 1.8.8.7 2001/09/27 20:00:31 kurt Exp $
#
# See slapd.conf(5) for details on configuration options.
# This file should NOT be world readable.
#
include		/etc/openldap/schema/core.schema
include		/etc/openldap/schema/cosine.schema
include		/etc/openldap/schema/inetorgperson.schema
include		/etc/openldap/schema/nis.schema
include		/etc/openldap/schema/samba.schema
include		/etc/openldap/schema/sendmail.schema

# Define global ACLs to disable default read access.

# Do not enable referrals until AFTER you have a working directory
# service AND an understanding of referrals.
#referral	ldap://root.openldap.org

pidfile		/usr/var/slapd.pid
argsfile	/usr/var/slapd.args

# Load dynamic backend modules:
# modulepath	/usr/lib/openldap/openldap
# moduleload	back_ldap.la
# moduleload	back_ldbm.la
# moduleload	back_passwd.la
# moduleload	back_shell.la

loglevel 0

# for sendmail allow downgrade to older protocol:
allow bind_v2

#
# Sample Access Control
#	Allow read access of root DSE
#	Allow self write access
#	Allow authenticated users read access
#	Allow anonymous users to authenticate
#
#access to dn="" by * read
#access to *
#	by self write
#	by users read
#	by anonymous auth
#
# if no access controls are present, the default is:
#	Allow read by all
#
# rootdn can always write!

# passwords are not readable but only usable for authentication,
# except for the user itself - there it is writable for password changing
access to dn="uid=[^,]+,ou=(machines|users|special users),dc=mylinux,dc=de"
	attrs=userPassword,sambaNTPassword,sambaLMPassword
	by anonymous auth
	by self write
	by * none

# no passwords are allowed for any other entries, therefore it is
# impossible to authenticate to an entry written or modified by
# a non-administrator (f.e. in an contact element)
access to *
	attrs=userPassword,sambaNTPassword,sambaLMPassword
	by * none

# the group attributes are writable for group LDAP Admin only
access to dn="cn=[^,]+,ou=groups,dc=mylinux,dc=de"
	attrs=memberUid,description
	by group="cn=ldap admins,dc=mylinux,dc=de" write
	by * read

# other group attributes (gidNumber etc.) are read only
access to dn="cn=[^,]+,ou=groups,dc=mylinux,dc=de"
	by * read

# no group membership allowed in elements outside directory groups
# therefore it is impossible for a non-administrator to redefine
# a system group
access to *
	attr=memberUid
	by * none

# access to other elements in this folder is none
access to dn=".+,ou=groups,dc=mylinux,dc=de"
	by * none

#######

# the samba attributes are non readable except for the user itself
access to dn="uid=[^,]+,ou=(machines|users|special users),dc=mylinux,dc=de"
	attrs=sambaPwdLastSet,sambaLogonTime,sambaLogoffTime,sambaKickoffTime,sambaPwdCanChange,sambaPwdMustChange,sambaNextGroupRid,sambaNextUserRid,sambaNextRid,sambaAcctFlags
	by self read
	by * none

access to *
	attrs=sambaPwdLastSet,sambaLogonTime,sambaLogoffTime,sambaKickoffTime,sambaPwdCanChange,sambaPwdMustChange,sambaNextGroupRid,sambaNextUserRid,sambaNextRid,sambaAcctFlags
	by * none

# the name, eMail and unix system attributes are readonly
access to dn="uid=[^,]+,ou=(machines|users|special users),dc=mylinux,dc=de"
	attrs=givenName,sn,uidNumber,gidNumber,homeDirectory,loginShell,mail,uid,displayName,cn
	by * read

# unix system attributes cannot be stored outside the
# machines/users/special users directories
# (the gidNumber for groups is readable because of another rule above)
access to *
	attrs=uidNumber,gidNumber,homeDirectory,loginShell,uid
	by * none

# except passwords (see "userPassword,..." above),
# samba attributes (see "sambaPwdLastSet,..." above)
# and unix attributes (see "givenName,..." above),
# the user/machine entries are writable for "self"
# and group "Data Maintain"
access to dn="uid=[^,]+,ou=(machines|users),dc=mylinux,dc=de"
	by self write
	by group="cn=data maintain,dc=mylinux,dc=de" write
	by * read

# except passwords (see "userPassword,..." above)
# and samba attributes (see "sambaPwdLastSet,..." above),
# the special user entries are read only
access to dn="uid=[^,]+,ou=special users,dc=mylinux,dc=de"
	by * read

# access to other elements in these folders is none
access to dn=".+,ou=(machines|users|special users),dc=mylinux,dc=de"
	by * none

#######

# the contact template is used only by group "Data Maintain"
# and edited only by group "LDAP Admins"
access to dn="cn=template,ou=contacts,dc=mylinux,dc=de"
        by group="cn=ldap admins,dc=mylinux,dc=de" write
        by group="cn=data maintain,dc=mylinux,dc=de" read
        by * none

# contacts are creatable/deletable for group "Data Maintain"
access to dn="ou=contacts,dc=mylinux,dc=de"
	attrs=entry,children
	by group="cn=data maintain,dc=mylinux,dc=de" write
	by * read

# contacts are writable for group "Data Maintain"
access to dn="cn=[^,]+,ou=contacts,dc=mylinux,dc=de"
	by group="cn=data maintain,dc=mylinux,dc=de" write
	by * read

# aliases are writable for group "LDAP Admins"
access to dn="(sendmailMTAMapName|sendmailMTAKey)=[^,]+,ou=aliases,dc=mylinux,dc=de"
	by group="cn=ldap admins,dc=mylinux,dc=de" write
	by * read

# access to other elements in these folders is none
access to dn=".+,ou=(contacts|aliases),dc=mylinux,dc=de"
	by * none

#######

# groups "LDAP Admins" / "Data Maintain" are writable for group "LDAP Admins"
access to dn="cn=(ldap admins|data maintain),dc=mylinux,dc=de"
	by group="cn=ldap admins,dc=mylinux,dc=de" write
	by * read

# allow read access for LDAP administrator entry - no
# interesting information is stored here
access to dn="cn=admin,dc=mylinux,dc=de"
	by * read

# first level entries: read access only
access to dn="ou=[^,]+,dc=mylinux,dc=de"
	by * read

# access to other first level elements is none:
access to dn=".+,dc=mylinux,dc=de"
	by * none

#######################################################################
# ldbm database definitions
#######################################################################

database	ldbm
suffix		"dc=mylinux,dc=de"
rootdn		"cn=admin,dc=mylinux,dc=de"
# Cleartext passwords, especially for the rootdn, should
# be avoid.  See slappasswd(8) and slapd.conf(5) for details.
# Use of strong authentication encouraged.
rootpw		fourier
# The database directory MUST exist prior to running slapd AND 
# should only be accessible by the slapd/tools. Mode 700 recommended.
directory	/usr/var/openldap-data
# Indices to maintain
index	objectClass,sendmailMTAKey,sendmailMTAMapName	eq
index	sendmailMTAHost,sendmailMTACluster	eq
index	cn,sn,uid,mail	pres,eq,sub

TLSCertificateFile /etc/openldap/certs/cert.pem
TLSCertificateKeyFile /etc/openldap/certs/key.pem
TLSCACertificateFile /etc/ssl/mylinux/cacert.pem 
EOF

# Export LDAP password to the myLinux usermanager:
echo -n "$LDAPPASSWD" > /root/usermanager/data/slappasswd
# Export LDAP password to Samba:
smbpasswd -w "$LDAPPASSWD"
